# -*- coding: utf-8 -*-
#
# Copyright (c) 2012 MailStore Software GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import sys
import base64
import json
import time
from mailstoreapi.errors import ApiError
from subprocess import Popen, PIPE

version = sys.version_info
if not type(version) == tuple:
    version = (version.major, )
    
if version[0] == 2:
    import urllib2 as urllib
    from urllib import urlencode
elif version[0] > 2:
    import urllib.request as urllib
    from urllib.parse import urlencode
else:
    raise ApiError("your python version is too old")

def replace(string, old,  new):
    if version[0] > 2:
        if type(string) != bytes:
            string = string.encode("utf-8")
        if type(old) != bytes:
            old = old.encode("utf-8")
        if type(new) != bytes:
            new = new.encode("utf-8")
    return string.replace(old, new)
    
class Client(object):
    def __init__(self, user="admin", password="admin",
                 host="localhost", port=8463, code="json"):
        self.host = host
        self.port = port
        self.user = user
        self.password = password
        self.code = code
        
        self.debug = False

    def call(self, method, params={}):
        """Calls a Command from the Server and returns the parsed json respond"""
        uri = "https://{0}:{1}/{2}/Invoke/{3}".format(self.host,
                                                      self.port,
                                                      self.code.upper(),
                                                      method)

        newParams = {}
        for key, value in params.items():
            if value is not None:
                newParams[key.encode("utf-8")] = value.encode("utf-8")
        params = newParams

        if params != {}:
            uri += "?"
            for key, value in params.items():
                uri += "{0}={1}&".format(urllib.quote(key), urllib.quote(value))
            uri = uri[:-1]
        
        if self.debug:
            print(uri)
            
        req = urllib.Request(uri)
        base64string = base64.b64encode(('{0}:{1}'.format(self.user, self.password)).encode("utf-8") ).replace(b'\n', b'')
        req.add_header(b"Authorization", b"Basic " + base64string)
        
        # post-get bug workaround:
        params = urlencode(params)
        params = params.encode("utf-8")
        req.add_data(params)
        
        try:
            rep = replace(urllib.urlopen(req).read(), '\xef\xbb\xbf', "")
            ret = self.parseJson(rep)
        except (urllib.HTTPError, urllib.URLError) as e:
            if hasattr(e,  "read"):
                e = e.read()
            raise ApiError(e)
        else:
            return ret
            
    def waitForToken(self, token, wait=0.5):
        """Waits until token has response and returns it."""
        recv = self.getToken(token)
        while isinstance(recv, dict) and recv["status"] == "running":
            time.sleep(wait)
            recv = self.getToken(token)
        return recv

    def getToken(self, token):
        """Checks if token has response and returns it or -1"""
        uri = "https://{0}:{1}{2}".format(self.host,
                                            self.port,
                                            token["statusUrl"])
        req = urllib.Request(uri)
        base64string = base64.b64encode(('{0}:{1}'.format(self.user, self.password)).encode("utf-8") ).replace(b'\n', b'')
        req.add_header(b"Authorization", b"Basic " + base64string)
        req.add_data(urlencode(token).encode("utf-8"))
        try:
            data = replace(urllib.urlopen(req).read(), '\xef\xbb\xbf', "")
            ret = self.parseJson(data.decode("utf-8"))
        except urllib.HTTPError as e:
            if self.debug:
                print("STATUSCODE:"+str(e.code) +" \r\n"+ str(e.read()))
            return -1
        return ret
        
    def getToken2(self, token):
        uri = "https://{0}:{1}{2}".format(self.host,
                                            self.port,
                                            token["statusUrl"])
        req = urllib.Request(uri)
        base64string = base64.b64encode(('{0}:{1}'.format(self.user, self.password)).encode("utf-8") ).replace(b'\n', b'')
        req.add_header(b"Authorization", b"Basic " + base64string)
        req.add_data(urlencode(token).encode("utf-8"))
        try:
            data = urllib.urlopen(req).read().replace('\xef\xbb\xbf', "")
            ret = self.parseJson(data.decode("utf-8"))
        except urllib.HTTPError as e:
            if self.debug:
                print("STATUSCODE:"+str(e.code) +" \r\n"+ str(e.read()))
            return -1
        return ret
        
    def waitForToken2(self, token, wait=0.5):
        """Waits until token has response and returns it."""
        recv = self.getToken2(token)
        while recv["status"] == "running":
            time.sleep(wait)
            recv = self.getToken(token)
        return recv

    def parseJson(self, jsoncode):
        jsoncode = replace(jsoncode, "\ufeff", "")
        if version[0] > 2:
            jsoncode = jsoncode.decode("utf-8")
        return json.loads(jsoncode)


